"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NS_RECEIVER = exports.NS_HEARTBEAT = exports.NS_CONNECTION = void 0;
const castv2_1 = require("castv2");
const messaging_1 = require("../../lib/messaging");
exports.NS_CONNECTION = "urn:x-cast:com.google.cast.tp.connection";
exports.NS_HEARTBEAT = "urn:x-cast:com.google.cast.tp.heartbeat";
exports.NS_RECEIVER = "urn:x-cast:com.google.cast.receiver";
class Session {
    constructor(host, port, appId, sessionId, referenceId) {
        this.host = host;
        this.port = port;
        this.appId = appId;
        this.sessionId = sessionId;
        this.referenceId = referenceId;
        this.channelMap = new Map();
        this.isSessionCreated = false;
        this.client = new castv2_1.Client();
        this.client.connect({ host, port }, this.onConnect.bind(this));
    }
    onConnect() {
        let transportHeartbeat;
        const sourceId = "sender-0";
        const destinationId = "receiver-0";
        this.clientConnection = this.client.createChannel(sourceId, destinationId, exports.NS_CONNECTION, "JSON");
        this.clientHeartbeat = this.client.createChannel(sourceId, destinationId, exports.NS_HEARTBEAT, "JSON");
        this.clientReceiver = this.client.createChannel(sourceId, destinationId, exports.NS_RECEIVER, "JSON");
        this.clientConnection.send({ type: "CONNECT" });
        this.clientHeartbeat.send({ type: "PING" });
        this.clientHeartbeatIntervalId = setInterval(() => {
            if (transportHeartbeat) {
                transportHeartbeat.send({ type: "PING" });
            }
            this.clientHeartbeat.send({ type: "PING" });
        }, 5000);
        this.clientReceiver.send({
            type: "LAUNCH",
            appId: this.appId,
            requestId: 1
        });
        this.clientReceiver.on("message", (message) => {
            if (message.type === "RECEIVER_STATUS") {
                this.sendMessage("shim:/session/updateStatus", message.status);
                if (message.status.applications) {
                    const receiverApp = message.status.applications[0];
                    const receiverAppId = receiverApp.appId;
                    this.app = receiverApp;
                    if (receiverAppId !== this.appId) {
                        this.sendMessage("shim:/session/stopped");
                        this.client.close();
                        clearInterval(this.clientHeartbeatIntervalId);
                        return;
                    }
                    if (!this.isSessionCreated) {
                        this.isSessionCreated = true;
                        this.transportId = this.app.transportId;
                        this.clientId =
                            `client-${Math.floor(Math.random() * 10e5)}`;
                        this.transportConnection = this.client.createChannel(this.clientId, this.transportId, exports.NS_CONNECTION, "JSON");
                        transportHeartbeat = this.client.createChannel(this.clientId, this.transportId, exports.NS_HEARTBEAT, "JSON");
                        this.transportConnection.send({ type: "CONNECT" });
                        this.sessionId = this.app.sessionId;
                        this.sendMessage("shim:/session/connected", {
                            sessionId: this.app.sessionId,
                            namespaces: this.app.namespaces,
                            displayName: this.app.displayName,
                            statusText: this.app.displayName
                        });
                    }
                }
            }
        });
    }
    messageHandler(message) {
        switch (message.subject) {
            case "bridge:/session/close":
                this.close();
                break;
            case "bridge:/session/impl_addMessageListener":
                this._impl_addMessageListener(message.data.namespace);
                break;
            case "bridge:/session/impl_sendMessage":
                this._impl_sendMessage(message.data.namespace, message.data.message, message.data.messageId);
                break;
            case "bridge:/session/impl_setReceiverMuted":
                this._impl_setReceiverMuted(message.data.muted, message.data.volumeId);
                break;
            case "bridge:/session/impl_setReceiverVolumeLevel":
                this._impl_setReceiverVolumeLevel(message.data.newLevel, message.data.volumeId);
                break;
            case "bridge:/session/impl_stop":
                this._impl_stop(message.data.stopId);
                break;
        }
    }
    createChannel(namespace) {
        if (!this.channelMap.has(namespace)) {
            this.channelMap.set(namespace, this.client.createChannel(this.clientId, this.transportId, namespace, "JSON"));
        }
    }
    close() {
        var _a, _b;
        (_a = this.clientConnection) === null || _a === void 0 ? void 0 : _a.send({ type: "CLOSE" });
        (_b = this.transportConnection) === null || _b === void 0 ? void 0 : _b.send({ type: "CLOSE" });
    }
    stop() {
        var _a;
        (_a = this.clientConnection) === null || _a === void 0 ? void 0 : _a.send({ type: "STOP" });
    }
    sendMessage(subject, data = {}) {
        messaging_1.sendMessage({
            subject,
            data,
            _id: this.referenceId
        });
    }
    _impl_addMessageListener(namespace) {
        var _a;
        this.createChannel(namespace);
        (_a = this.channelMap.get(namespace)) === null || _a === void 0 ? void 0 : _a.on("message", (data) => {
            this.sendMessage("shim:/session/impl_addMessageListener", {
                namespace,
                data: JSON.stringify(data)
            });
        });
    }
    _impl_sendMessage(namespace, message, messageId) {
        let error = false;
        try {
            if (typeof message === "string") {
                message = JSON.parse(message);
            }
            this.createChannel(namespace);
            this.channelMap.get(namespace).send(message);
        }
        catch (err) {
            error = true;
        }
        this.sendMessage("shim:/session/impl_sendMessage", {
            messageId,
            error
        });
    }
    _impl_setReceiverMuted(muted, volumeId) {
        let error = false;
        try {
            this.clientReceiver.send({
                type: "SET_VOLUME",
                volume: { muted },
                requestId: 0
            });
        }
        catch (err) {
            error = true;
        }
        this.sendMessage("shim:/session/impl_setReceiverMuted", {
            volumeId,
            error
        });
    }
    _impl_setReceiverVolumeLevel(newLevel, volumeId) {
        let error = false;
        try {
            this.clientReceiver.send({
                type: "SET_VOLUME",
                volume: { level: newLevel },
                requestId: 0
            });
        }
        catch (err) {
            error = true;
        }
        this.sendMessage("shim:/session/impl_setReceiverVolumeLevel", {
            volumeId,
            error
        });
    }
    _impl_stop(stopId) {
        let error = false;
        try {
            this.clientReceiver.send({
                type: "STOP",
                sessionId: this.sessionId,
                requestId: 0
            });
        }
        catch (err) {
            error = true;
        }
        this.client.close();
        clearInterval(this.clientHeartbeatIntervalId);
        this.sendMessage("shim:/session/impl_stop", {
            stopId,
            error
        });
    }
}
exports.default = Session;
