"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopMediaServer = exports.startMediaServer = exports.mediaServer = void 0;
const fs_1 = __importDefault(require("fs"));
const http_1 = __importDefault(require("http"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const stream_1 = __importDefault(require("stream"));
const mime_types_1 = __importDefault(require("mime-types"));
const messaging_1 = require("../lib/messaging");
const subtitles_1 = require("../lib/subtitles");
function startMediaServer(filePath, port) {
    return __awaiter(this, void 0, void 0, function* () {
        if (exports.mediaServer === null || exports.mediaServer === void 0 ? void 0 : exports.mediaServer.listening) {
            yield stopMediaServer();
        }
        let fileDir;
        let fileName;
        let fileSize;
        try {
            const stat = yield fs_1.default.promises.lstat(filePath);
            if (stat.isFile()) {
                fileDir = path_1.default.dirname(filePath);
                fileName = path_1.default.basename(filePath);
                fileSize = stat.size;
            }
            else {
                console.error("Error: Media path is not a file.");
                messaging_1.sendMessage({
                    subject: "mediaCast:/mediaServer/error"
                });
                return;
            }
        }
        catch (err) {
            console.error("Error: Failed to find media path.");
            messaging_1.sendMessage({
                subject: "mediaCast:/mediaServer/error"
            });
            return;
        }
        const contentType = mime_types_1.default.lookup(filePath);
        if (!contentType) {
            console.error("Error: Failed to find media type.");
            messaging_1.sendMessage({
                subject: "mediaCast:/mediaServer/error"
            });
            return;
        }
        const subtitles = new Map();
        try {
            const dirEntries = yield fs_1.default.promises.readdir(fileDir, { withFileTypes: true });
            for (const dirEntry of dirEntries) {
                if (dirEntry.isFile()
                    && mime_types_1.default.lookup(dirEntry.name) === "application/x-subrip") {
                    subtitles.set(dirEntry.name, yield subtitles_1.convertSrtToVtt(path_1.default.join(fileDir, dirEntry.name)));
                }
            }
        }
        catch (err) { }
        exports.mediaServer = http_1.default.createServer((req, res) => __awaiter(this, void 0, void 0, function* () {
            if (!req.url) {
                return;
            }
            if (req.url.startsWith("/")) {
                req.url = req.url.slice(1);
            }
            switch (req.url) {
                case fileName: {
                    const { range } = req.headers;
                    if (range) {
                        const bounds = range.substring(6).split("-");
                        const start = parseInt(bounds[0]);
                        const end = bounds[1]
                            ? parseInt(bounds[1]) : fileSize - 1;
                        res.writeHead(206, {
                            "Accept-Ranges": "bytes",
                            "Content-Range": `bytes ${start}-${end}/${fileSize}`,
                            "Content-Length": (end - start) + 1,
                            "Content-Type": contentType
                        });
                        fs_1.default.createReadStream(filePath, { start, end }).pipe(res);
                    }
                    else {
                        res.writeHead(200, {
                            "Content-Length": fileSize,
                            "Content-Type": contentType
                        });
                        fs_1.default.createReadStream(filePath).pipe(res);
                    }
                    break;
                }
                default: {
                    if (subtitles.has(req.url)) {
                        const vttSource = subtitles.get(req.url);
                        const vttStream = stream_1.default.Readable.from(vttSource);
                        res.setHeader("Access-Control-Allow-Origin", "*");
                        vttStream.pipe(res);
                    }
                    break;
                }
            }
        }));
        exports.mediaServer.on("listening", () => {
            let localAddress = "";
            const ifaces = Object.values(os_1.default.networkInterfaces());
            for (const iface of ifaces) {
                const matchingIface = iface === null || iface === void 0 ? void 0 : iface.find(details => details.family === "IPv4" && !details.internal);
                if (matchingIface) {
                    localAddress = matchingIface.address;
                }
            }
            if (!localAddress) {
                console.error("Failed to get local address.");
                messaging_1.sendMessage({
                    subject: "mediaCast:/mediaServer/error"
                });
                stopMediaServer();
                return;
            }
            messaging_1.sendMessage({
                subject: "mediaCast:/mediaServer/started",
                data: {
                    mediaPath: fileName,
                    subtitlePaths: Array.from(subtitles.keys()),
                    localAddress
                }
            });
        });
        exports.mediaServer.on("close", () => messaging_1.sendMessage({
            subject: "mediaCast:/mediaServer/stopped"
        }));
        exports.mediaServer.on("error", () => messaging_1.sendMessage({
            subject: "mediaCast:/mediaServer/error"
        }));
        exports.mediaServer.listen(port);
    });
}
exports.startMediaServer = startMediaServer;
function stopMediaServer() {
    if (exports.mediaServer === null || exports.mediaServer === void 0 ? void 0 : exports.mediaServer.listening) {
        exports.mediaServer.close();
        exports.mediaServer = undefined;
    }
}
exports.stopMediaServer = stopMediaServer;
